'use strict';

var page = require('page');
var utils = require('./src/utils');

module.exports = function(Vue) {
    var component = Vue.directive('_component');
    var router = require('./src/router')(Vue, page, utils);
    var go = require('./src/go')(Vue, utils);
    var _ = Vue.util;
    var parsers = Vue.parsers;

    // 缓存view.
    var viewCache = {};
    var viewGroup = {};

    _.extend(router, component);
    _.extend(router, {
        bind: function() {
            this._isDynamicLiteral = true;

            component.bind.call(this);

            this.init();
        },

        /*
            copied/pasted from https://github.com/yyx990803/vue/blob/master/src/directives/component.js
            pass data properties to the view
            TODO: what the best way to pass data?
        */
        build: function() {
            if (this.keepAlive) {
                var cached = this.cache[this.ctorId];

                if (cached) {
                    return cached;
                }
            }

            var vm = this.vm;
            var el = parsers.template.clone(this.el);
            var location = this._location;
            var router = {
                params: location.params,
                queryParams: location.queryParams
            };

            if (this._states) {

            }

            var data = _.extend(this._routerData.data || {}, {
                $router: router
            });

            if (this.Ctor) {
                var child = vm.$addChild({
                    el: el,
                    _asComponent: true,
                    data: function() {
                        return data;
                    }
                }, this.Ctor);

                if (this.keepAlive) {
                    this.cache[this.ctorId] = child;
                }

                this.child = child;

                return child;
            }
        },

        // if option "wait" passed to the route, wait for readyEvent emitted
        update: function(value) {
            var wait = this._routerData.wait != null ? this._routerData.wait : this._options.wait;

            if (wait) {
                this.readyEvent = this._options.readyEvent || 'dataLoaded';
            }

            component.update.call(this, value);

            if (this.readyEvent) {
                this.child.$once(this.readyEvent, function(e) {
                    this._callHook('onLoaded');
                }.bind(this));

                // set readyEvent to null, because not every route can have "wait" option
                this.readyEvent = null;
            }

        }
    });

    _.extend(viewGroup, component);
    _.extend(viewGroup, router);
    _.extend(viewGroup, {
        bind: function() {
            this._isDynamicLiteral = true;

            component.bind.call(this);

            this.init();
        },

        build: function() {
            this._el = parsers.template.clone(this.el);

            return this.create();
        },

        create: function() {

            var vm = this.vm;
            var location = this._location;
            var router = {
                params: location.params,
                queryParams: location.queryParams
            };

            if (this._states) {
                router.state = location.state;
                router.isState = function(stateName) {
                    return router.state === stateName;
                }
            }

            var data = _.extend(this._routerData.data || {}, {
                $router: router
            });

            // 如果已经存在则直接返回
            if (viewCache[router.state]) {
                return viewCache[router.state];
            }

            var _el = document.createElement('div');
            this._el.appendChild(_el);

            if (this.Ctor) {
                var child = vm.$addChild({
                    el: _el,
                    _asComponent: true,
                    data: function() {
                        return data;
                    }
                }, this.Ctor);

                viewCache[router.state] = child;

                return child;
            }
        },

        update: function(value) {
            if (this._routerData.wait === true) {
                this.readyEvent = this._options.readyEvent || 'dataLoaded';
            }

            var hasComponent = viewCache[value];
            if (!hasComponent) {
                this.build();
            }

            component.update.call(this, value);

            this.readyEvent = null;
        }
    });

    Vue.directive('view', router);
    Vue.directive('go', go);
    // 设置view group
    Vue.directive('view-group', viewGroup)
};
